// package anonym_put -- анонимное помещение ключа в хранилище
package anonym_put

import (
	"fmt"
	"log"

	"github.com/gofiber/fiber/v2"

	"git.p78su.freemyip.com/svi/gostore/pkg/types"
)

// AnonymPut -- анонимное помещение ключа в хранилище
type AnonymPut struct {
	serv  types.IService
	store types.IStore
}

// NewAnonymPut -- возвращает анонимное помещение ключа в хранилище
func NewAnonymPut(servHttp types.IServHttp, router fiber.Router) (*AnonymPut, error) {
	{ // Предусловия
		if servHttp == nil {
			return nil, fmt.Errorf("NewAnonymPut(): IServHttp==nil")
		}
		if router == nil {
			return nil, fmt.Errorf("NewAnonymPut(): router==nil")
		}
	}
	sf := &AnonymPut{
		serv:  servHttp.Service(),
		store: servHttp.Service().Store(),
	}
	router.Get("/put", sf.get)
	router.Post("/put_check_key", sf.putCheckKey)
	router.Post("/put", sf.post)
	return sf, nil
}

// PutRequest -- добавляет значение в анонимное хранилище
type PutRequest struct {
	Key_   string `json:"key" form:"key"` // Ключ для хранилища
	Value_ string `json:"val" form:"val"` // Значение для хранилища
}

// Проверяет допустимость ключа для анонимного хранилища
func (sf *AnonymPut) putCheckKey(ctx *fiber.Ctx) error {
	strBody := string(ctx.Body())
	log.Println("AnonymPut.putCheckKey():", strBody)
	req := &PutRequest{}
	err := ctx.BodyParser(req)
	if err != nil {
		return ctx.SendString("AnonymPut.putCheckKey(): " + err.Error())
	}
	if req.Key_ == "" {
		return ctx.SendString("AnonymPut.putCheckKey(): ключ не может быть пустым")
	}
	return ctx.SendString("AnonymPut.putCheckKey(): годный ключ")
}

// Возвращает страницу анонимного раздела
func (sf *AnonymPut) get(ctx *fiber.Ctx) error {
	return ctx.Render("anonym_put", fiber.Map{})
}

// Устанавливает значение в анонимное хранилище
func (sf *AnonymPut) post(ctx *fiber.Ctx) error {
	req := &PutRequest{}
	if err := ctx.BodyParser(req); err != nil {
		return ctx.Render("anonym_put", fiber.Map{
			"res": fmt.Sprintf("AnonymPut.post(): при парсинге формы, err=%v", err),
		})
	}
	if req.Key_ == "" {
		return ctx.Render("anonym_put", fiber.Map{
			"res": "AnonymPut.post(): ключ не может быть пустым",
		})
	}
	err := sf.store.Put("anon/"+req.Key_, []byte(req.Value_))
	if err != nil {
		return ctx.Render("anonym_put", fiber.Map{
			"res": fmt.Sprintf("AnonymPut.post(): при размещении ключа %q в хранилище, err=%v", req.Key_, err),
		})
	}
	return ctx.Render("anonym_put", fiber.Map{
		"res": fmt.Sprintf("AnonymPut.post(): key=%q, ok", req.Key_),
	})
}
